#include <string_inspector.h>
#include <gtk/gtk.h>
#include <gtkutils.h>
#include <jmp.h>
#include <heap_dump.h>

enum {
    SI_TEXT,
    SI_OBJ,
    SIN_COLUMNS
};


static void add_string (obj* o, GtkListStore* clist) {
    down_link* dl;
    get_instance_info (o);
    dl = get_last_down_link ();
    while (dl != NULL) {
	glong items_read;
	glong items_written;
	GError* err = NULL;
	gchar* converted_text;
	GtkTreeIter iter;

	switch (dl->type) {
	case JVMPI_GC_PRIM_ARRAY_DUMP:
	    converted_text = 
		g_utf16_to_utf8 ((gunichar2*)dl->value.txt, dl->pos, &items_read, 
				 &items_written, &err);
	    if (err != NULL) {
		fprintf (stderr, "Unable to convert text: %s\n", err->message);	
		g_error_free (err);
		break;
	    }
	    gtk_list_store_append (clist, &iter);
	    gtk_list_store_set (clist, &iter, 
				SI_TEXT, converted_text,
				SI_OBJ, o,
				-1);
	    g_free (converted_text);
	    break;
	default:
	    fprintf (stderr, "Odd type when dumping strings: %d\n", dl->type);
	}
	dl = dl->next;	
    }
    free_last_down_link ();
}

static gint sort_by_text (GtkTreeModel *model,
			  GtkTreeIter *a,
			  GtkTreeIter *b,
			  gpointer user_data) {
    gchar* cn1;
    gchar* cn2;
    gtk_tree_model_get (model, a, SI_TEXT, &cn1, -1);
    gtk_tree_model_get (model, b, SI_TEXT, &cn2, -1); 
    return g_strcasecmp (cn1, cn2);
}

void inspect_strings () {
    GtkListStore* clist;
    GtkWidget* scrolled_window;
    GtkWidget* tree;
    GtkWidget* strings_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_window_set_title (GTK_WINDOW (strings_window), _("Current strings (char[]) in memory"));
    clist = gtk_list_store_new (SIN_COLUMNS, G_TYPE_STRING, G_TYPE_POINTER);
    for_each_string ((string_callback)add_string, clist);
    scrolled_window = gtk_scrolled_window_new (NULL, NULL);
    gtk_container_add (GTK_CONTAINER (strings_window), scrolled_window);
    tree = gtk_tree_view_new_with_model (GTK_TREE_MODEL (clist));
    add_column (tree, _("Text"), SI_TEXT, (gpointer)tree, NULL, 300, 0);
    gtk_tree_view_set_headers_clickable (GTK_TREE_VIEW (tree), TRUE);
    gtk_container_add (GTK_CONTAINER (scrolled_window), tree);
    
    gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (clist), 0, GTK_SORT_ASCENDING);
    gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (clist), 0, 
				     sort_by_text, NULL, NULL);
    gtk_tree_sortable_sort_column_changed (GTK_TREE_SORTABLE (clist));
    
    gtk_widget_set_usize (strings_window, 400, 200);
    gtk_widget_show_all (strings_window);           
}

